#!/usr/bin/perl

use strict;
my $usage=q{
 splitbyNs.pl <input_fasta> <output_name> [<multiple_files>]
 
 Looks for regions of at least 300Ns and splits the input sequence there.
 
};

my ($fastafile,$outputfile,$multiple)=@ARGV;

my $n=1;
$/="\n>";

open(F,$fastafile);

while(<F>) {
    s/^>//;
    chomp;
    next unless $_;
    my ($name, $descr, $seq)=(m/^(\S+)[ \t\x01]*(.*?)\n(.+)/s);
    die "ERROR 21: Wrong FASTA format: .$_." unless $name && $seq;
    $seq=~tr/\t \n\r//d;
    $seq=lc($seq);
    $n=processseq($name,$seq,$outputfile,$n,$multiple);
}

close(F);


sub processseq {
    my($name,$seq,$output,$n,$multiple)=@_;

    my $laststart=0;

    my $ns='n' x 300;

    my $count=1;
    my $pos;
    my $newout;
    while(($pos=index($seq,$ns,$laststart))>0) {
	my $newseq=substr($seq,$laststart,$pos-$laststart);
	if ($multiple) {
	    $newout=$output.".".$n;
	    $n++;
	    open(O,">$newout");
	}
	else {
	    open(O,">>$output");
	}
	print O ">$name";
	if($multiple) { 
	    print O " $laststart\n";
	}
	else {
	    print O "_$count $laststart\n";
	}
	print O "$newseq\n";
	close(O);
	$count++;
	$laststart=skipns($seq,$pos);
    }
    if($multiple) {
	$newout=$output.".".$n;
	$n++;
	open(O,">$newout");
    }
    else {
	open(O,">>$output");
    }
    my $newseq=substr($seq,$laststart);
    print O ">$name";
    if($multiple) { 
	print O " $laststart\n";
    }
    else {
	print O "_$count $laststart\n";
    }
    print O "$newseq\n";
    close(O);
    
}

sub skipns {
    my($seq,$pos)=@_;

    while(substr($seq,$pos,1) eq "n") { $pos++;}

    return($pos);
}
