#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*
 *  long_spanning_reads.cpp
 *  TopHat
 *
 *  Created by Cole Trapnell on 2/5/09.
 *  Copyright 2009 Cole Trapnell. All rights reserved.
 *
 */

#include <cassert>
#include <cstdio>
#include <vector>
#include <string>
#include <map>
#include <algorithm>
#include <set>
#include <cstdlib>
#include <cstring>
#include <bitset>

#include <iostream>
#include <fstream>
#include <sstream>
#include <getopt.h>

#include "common.h"
#include "bwt_map.h"
#include "tokenize.h"
#include "reads.h"

using namespace seqan;
using namespace std;

void print_usage()
{
    fprintf(stderr, "Usage:   bwt2sam --sam-header <sam_header.sam> <bwt_file>\n");
}

int multi_closure = 0;
int anchor_too_short = 0;
int gap_too_short = 0;
bool from_bowtie=false;

void print_segment_hits(string fname, HitStream& hitstream,
    ReadTable& unmapped_reads,
    RefSequenceTable& rt) {
  HitsForRead curr_hit_group;
  fname+=".bam";
  GBamWriter bam_writer(fname.c_str(), sam_header.c_str());
  while (true) {
    hitstream.next_read_hits(curr_hit_group);
    if (curr_hit_group.hits.size()==0) break;
    for (size_t i = 0; i < curr_hit_group.hits.size(); i++) {
        BowtieHit& bh = curr_hit_group.hits[i];
        const char* ref_name = rt.get_name(bh.ref_id());
        string read_name(""); //stoopid STL
        str_appendInt(read_name, bh.insert_id());
        print_hit(stdout, read_name.c_str(), curr_hit_group.hits[i], ref_name,
              bh.seq().c_str(),bh.qual().c_str(),
              hitstream.fromBowtie());
        print_bamhit(bam_writer, read_name.c_str(), curr_hit_group.hits[i], ref_name,
              bh.seq().c_str(),bh.qual().c_str(),hitstream.fromBowtie());
        }
    }
}


//void driver(GBamWriter& bam_writer, vector<FZPipe>& seg_files)
//void driver(vector<FZPipe>& seg_files)
void driver(string& seg_file_name)
{

  RefSequenceTable rt(sam_header, true);
  //fprintf (stderr, "Loading reference sequences...\n");
  //get_seqs(ref_stream, rt, true, false);
  //  fprintf (stderr, "        reference sequences loaded.\n");
  ReadTable it;

  bool need_seq = true;
  bool need_qual = true;

  HitFactory* fac = NULL;
  string fext=getFext(seg_file_name);
  from_bowtie=true;
  if (fext=="sam")
         fac = new SAMHitFactory(it, rt);
    else if (fext=="bam")
         fac = new BAMHitFactory(it, rt);
       else {
         fac= new BowtieHitFactory(it, rt);
         //from_bowtie=true;
         }
  //HitStream hs(seg_files[i].file, fac, false, false, false, need_seq, need_qual);
  HitStream hs(seg_file_name, fac, false, false, false, need_seq, need_qual, from_bowtie);

  print_segment_hits(seg_file_name, hs, it, rt);

  delete fac;

} //driver

int main(int argc, char** argv)
{
  fprintf(stderr, "bwt2sam v%s (%s)\n", PACKAGE_VERSION, SVN_REVISION);
  fprintf(stderr, "--------------------------------------------\n");

  int parse_ret = parse_options(argc, argv, print_usage);
  if (parse_ret)
    return parse_ret;

  if(optind >= argc)
    {
      print_usage();
      return 1;
    }

  string segment_file_name = argv[optind++];

  string unzcmd=getUnpackCmd(segment_file_name,false);
  /*
  vector<FZPipe> segment_files;
  for (size_t i = 0; i < segment_file_names.size(); ++i)
    {
      fprintf(stderr, "Opening %s for reading\n",
      segment_file_names[i].c_str());
      FZPipe seg_file(segment_file_names[i], unzcmd);
      if (seg_file.file == NULL)
        {
        err_die("Error: cannot open %s for reading\n",
              segment_file_names[i].c_str());
        }
      segment_files.push_back(seg_file);
    }

  //writeSamHeader(stdout);
  //GBamWriter bam_writer("-", sam_header.c_str());
  //driver(bam_writer, ref_stream, juncs_files, insertions_files, deletions_files, spliced_segment_files, segment_files, reads_file);
  driver(segment_files);
  */
  driver(segment_file_name);
  return 0;
}
